require_relative 'player'
require_relative 'die'
require_relative 'loaded_die'
require_relative 'game_turn'
require_relative 'treasure_trove'

module StudioGame
  class Game 
    attr_accessor :title
    attr_reader :players


    def initialize(title,players=[])
      @title = title 
      @players = players
    end

    def add_player(player)
      @players.push(player)
    end


    def play(rounds)
      puts "\nThere are #{@players.size} players in #{title}:"

      each_player do |p|
        puts p
      end

      tr = TreasureTrove::TREASURES 
      puts "\nThere are #{tr.size} treasures to be found:\n"

      tr.each do |t|
        puts "A #{t.name} is worth #{t.points} points\n"
      end

      1.upto(rounds) do |i|
        puts "\nRound #{i}\n"

        each_player do |p|
          GameTurn.take_turn(p)
        end
        # break if yield
      end

    end

    def statistics
      strong, wimpy = @players.partition {|n| n.strong?}
      puts "\n\n\n#{@title} Statistics:\n\n"
      puts "#{strong.size} strong player#{"s"if strong.size != 1}:\n"
      strong.each do |p|
        puts "#{p.name} (#{p.health})\n"
      end
      puts "\n"

      puts "#{wimpy.size} wimpy player#{"s"if strong.size != 1}:\n"
      wimpy.each do |p|
        puts "#{p.name} (#{p.health})\n"
      end

      print_high_scores
      each_player { |p| p.print_points }
      puts "\nThe total combined score is #{total_points}"
    end

    def high_score_entry(player)
      "#{player.name.ljust(30,'.')} #{player.score}\n"
    end

    def each_player
      sorted_players = @players.sort
      sorted_players.each { |sp| yield sp }
    end


    def print_high_scores
      puts "\n#{@title} High Scores:\n"
      each_player do |sp|
        puts high_score_entry(sp)
      end

    end

    def read_csv(file)
      File.new(file).readlines.each do |line|
        x, y = line.split(",")
        yield(x,y)
      end
    end

    def load_players(file)
      read_csv(file) do |name, health|
        player = Player.new(name,Integer(health))
        add_player(player)
      end
    end


    def total_points
      @players.reduce(0) {|sum, p| sum + p.score}
    end

    def save_high_scores(file_name="high_scores.txt")
      File.open(file_name,"w") do |file|
        each_player do |pl|
          file.puts high_score_entry(pl)
        end
      end

    end




  end
end
