require_relative 'treasure_trove'
require_relative 'playable'

module StudioGame
  class Player
    include Playable

    attr_accessor :health, :treasures, :name


    def initialize(name, health=100)
      @name = name.capitalize
      @health = health
      @found_treasures = Hash.new(0) 
    end

    def to_s 
      "I'm #{@name.capitalize} with a health of #{@health} and a score of #{score.to_s}."
    end



    def pass
      puts "#{@name} got passed."
    end

    def found_treasure(t)
      @found_treasures[t.name] += t.points
      puts "#{@name} found a #{t.name} worth #{t.points}"
    end

    def points
      @found_treasures.values.reduce(0) { |sum, v| sum + v }
    end

    def score
      @health + @name.length + self.points 
    end

    def <=>(other)
      other.score <=> self.score
    end


    def each_found_treasure
      @found_treasures.each do |key, value| 
        yield( Treasure.new(key,value)) 
      end
    end

    def print_points
      puts "\n#{@name}'s point totals:"
      self.each_found_treasure do |treasure|
        puts "#{treasure.points} total #{treasure.name} points"
      end
      puts "#{self.points} grand total points"
    end



  end

end
